/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
#include <config.h>

#include "gir-tray-icon.h"
#include <gnome.h>
#include "eggtrayicon.h"

enum {
	STATE_DISCONNECTED,
	STATE_INACTIVE,
	STATE_ACTIVE,
	NUM_STATE,
};

struct _GirTrayIcon {
	EggTrayIcon *icon;

	int state;

	/* icon image */
	GtkImage *icon_image;
	struct {
		GdkPixbuf *pixbuf;
		GdkPixbuf *pixbuf_scaled;
	} pixbufs[NUM_STATE];
	int pixbuf_scaled_size;

	guint active_timeout;

	/* menu */
	GtkMenu *menu;
	GtkCheckMenuItem *menu_item_connect;

	void (*connect_callback) (GirTrayIcon *tray, gboolean activated,
				  gpointer data);
	void (*about_callback) (GirTrayIcon *tray, gpointer data);
	void (*prefs_callback) (GirTrayIcon *tray, gpointer data);
	void (*quit_callback) (GirTrayIcon *tray, gpointer data);
	gpointer connect_callback_data;
	gpointer prefs_callback_data;
	gpointer about_callback_data;
	gpointer quit_callback_data;
};



/*
 * stolen code from nabi src/ui.c (http://nabi.kldp.net)
 */
static void
nabi_menu_position_func(GtkMenu *menu,
			gint *x, gint *y,
			gboolean *push_in,
			gpointer data)
{
    GdkWindow *window;
    GdkScreen *screen;
    gint width, height, menu_width, menu_height;
    gint screen_width, screen_height;

    window = GDK_WINDOW(GTK_WIDGET(data)->window);
    screen = gdk_drawable_get_screen(GDK_DRAWABLE(window));
    screen_width = gdk_screen_get_width(screen);
    screen_height = gdk_screen_get_height(screen);
    gdk_window_get_origin(window, x, y);
    gdk_drawable_get_size(window, &width, &height);
    if (!GTK_WIDGET_REALIZED(menu))
	gtk_widget_realize(GTK_WIDGET(menu));
    gdk_drawable_get_size(GDK_WINDOW(GTK_WIDGET(menu)->window),
			  &menu_width, &menu_height);

    if (*y + height < screen_height / 2)
	*y += height;
    else
	*y -= menu_height;

    if (*x + menu_width > screen_width)
	*x = screen_width - menu_width;
}

static gboolean
icon_press (GirTrayIcon *tray_icon, GdkEventButton *event)
{
	GtkMenu *menu = tray_icon->menu;

	if (event->button == 3) {
		gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
				nabi_menu_position_func, tray_icon->icon,
				event->button, event->time);
		return TRUE;
	}
	return FALSE;
}

static gboolean
icon_release (GirTrayIcon *tray_icon, GdkEventButton *event)
{
	GtkMenu *menu = tray_icon->menu;

	if (event->button == 3)	{
		gtk_menu_popdown (GTK_MENU (menu));
		return FALSE;
	}
	return TRUE;
}

static void
icon_size_allocate (GirTrayIcon *tray_icon, GtkAllocation *allocation)
{
	int size;
	GtkOrientation orientation;
	int i;
	int state;

	if (! GTK_WIDGET_REALIZED (tray_icon->icon))
		return;

	orientation = egg_tray_icon_get_orientation (tray_icon->icon);
	if (orientation == GTK_ORIENTATION_HORIZONTAL)
		size = allocation->height;
	else
		size = allocation->width;

	if (size == tray_icon->pixbuf_scaled_size)
		return;

	for (i = 0; i < NUM_STATE; i++) {
		if (tray_icon->pixbufs[i].pixbuf_scaled)
			gdk_pixbuf_unref (tray_icon->pixbufs[i].pixbuf_scaled);
		tray_icon->pixbufs[i].pixbuf_scaled =
			gdk_pixbuf_scale_simple (tray_icon->pixbufs[i].pixbuf,
						 size, size,
						 GDK_INTERP_BILINEAR);
	}
	tray_icon->pixbuf_scaled_size = size;

	state = tray_icon->state;
	gtk_image_set_from_pixbuf (tray_icon->icon_image,
				   tray_icon->pixbufs[state].pixbuf_scaled);
}



/*
 * menu callbacks
 */

static void
menu_connect (GtkCheckMenuItem *check_item, gpointer data)
{
	GirTrayIcon *tray_icon = (GirTrayIcon*)data;
	gboolean active;

	active = gtk_check_menu_item_get_active (check_item);
	if (tray_icon->connect_callback)
		tray_icon->connect_callback (tray_icon, active,
					     tray_icon->connect_callback_data);
}

static void
menu_preferences (GirTrayIcon *tray_icon)
{
	if (tray_icon->prefs_callback)
		tray_icon->prefs_callback (tray_icon,
					   tray_icon->prefs_callback_data);
}

static void
menu_about (GirTrayIcon *tray_icon)
{
	if (tray_icon->about_callback)
		tray_icon->about_callback (tray_icon,
					   tray_icon->about_callback_data);
}

static void
menu_quit (GirTrayIcon *tray_icon)
{
	if (tray_icon->quit_callback)
		tray_icon->quit_callback (tray_icon,
					   tray_icon->about_callback_data);
}



static gboolean
icon_active_timeout (gpointer data)
{
	GirTrayIcon *tray_icon = (GirTrayIcon *)data;

	if (tray_icon->state == STATE_ACTIVE) {
		tray_icon->state = STATE_INACTIVE;
		tray_icon->active_timeout = 0;
		gtk_image_set_from_pixbuf
			(tray_icon->icon_image,
			 tray_icon->pixbufs[STATE_INACTIVE].pixbuf_scaled);
	}
	return FALSE;
}



static void
create_menu (GirTrayIcon *tray_icon)
{
	GtkWidget *menu, *item;

	menu = gtk_menu_new ();
	tray_icon->menu = GTK_MENU (menu);

	item = gtk_check_menu_item_new_with_mnemonic (_("_Connect"));
	g_signal_connect (G_OBJECT (item), "activate",
			  G_CALLBACK (menu_connect), tray_icon);
	gtk_widget_show (item);
	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
	tray_icon->menu_item_connect = GTK_CHECK_MENU_ITEM (item);
	
	item = gtk_image_menu_item_new_from_stock (GTK_STOCK_PREFERENCES,
						   NULL);
	g_signal_connect_swapped (G_OBJECT (item), "activate",
				  G_CALLBACK (menu_preferences), tray_icon);
	gtk_widget_show (item);
	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);

	item = gtk_image_menu_item_new_from_stock (GNOME_STOCK_ABOUT, NULL);
	g_signal_connect_swapped (G_OBJECT (item), "activate",
				  G_CALLBACK (menu_about), tray_icon);
	gtk_widget_show (item);
	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);

	item = gtk_image_menu_item_new_from_stock (GTK_STOCK_QUIT,
						   NULL);
	g_signal_connect_swapped (G_OBJECT (item), "activate",
				  G_CALLBACK (menu_quit), tray_icon);
	gtk_widget_show (item);
	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
}



GirTrayIcon *
gir_tray_icon_new (void)
{
	GirTrayIcon *tray_icon;
	GtkWidget *box;

	tray_icon = g_new0 (GirTrayIcon, 1);

	tray_icon->icon = egg_tray_icon_new ("gir tray");
	create_menu (tray_icon);

	box = gtk_event_box_new ();
	g_signal_connect_swapped (G_OBJECT (box), "button_press_event",
				  G_CALLBACK (icon_press), tray_icon);
	g_signal_connect_swapped (G_OBJECT (box), "button_release_event",
				  G_CALLBACK (icon_release), tray_icon);
	g_signal_connect_swapped (G_OBJECT (box), "size_allocate",
				  G_CALLBACK (icon_size_allocate), tray_icon);

	tray_icon->icon_image = GTK_IMAGE (gtk_image_new());
	tray_icon->pixbufs[STATE_DISCONNECTED].pixbuf =
		gdk_pixbuf_new_from_file (IMAGEDIR "/wave-disconnected.png",
					  NULL);
	tray_icon->pixbufs[STATE_INACTIVE].pixbuf =
		gdk_pixbuf_new_from_file (IMAGEDIR "/wave-inactive.png", NULL);
	tray_icon->pixbufs[STATE_ACTIVE].pixbuf =
		gdk_pixbuf_new_from_file (IMAGEDIR "/wave-active.png", NULL);
	tray_icon->pixbuf_scaled_size = G_MAXINT; /* scaled later */

	gtk_container_add (GTK_CONTAINER (box),
			   GTK_WIDGET (tray_icon->icon_image));

	gtk_container_add (GTK_CONTAINER (tray_icon->icon), box);
	gtk_widget_show_all (GTK_WIDGET (tray_icon->icon));

	return tray_icon;
}

void
gir_tray_icon_free (GirTrayIcon *tray_icon)
{
	int i;

	for (i = 0; i < NUM_STATE; i++) {
		gdk_pixbuf_unref (tray_icon->pixbufs[i].pixbuf);
		gdk_pixbuf_unref (tray_icon->pixbufs[i].pixbuf_scaled);
	}
	gtk_widget_destroy (GTK_WIDGET (tray_icon->menu));
	gtk_widget_destroy (GTK_WIDGET (tray_icon->icon));
	g_free (tray_icon);
}


			     
void
gir_tray_icon_set_connect_callback (GirTrayIcon *tray_icon,
				    void (*cb)(GirTrayIcon *tray,
					       gboolean     activated,
					       gpointer     data),
				    gpointer data)
{
	tray_icon->connect_callback = cb;
	tray_icon->connect_callback_data = data;
}

void
gir_tray_icon_set_about_callback (GirTrayIcon *tray_icon,
				  void (*cb)(GirTrayIcon *tray,
					     gpointer     data),
				  gpointer data)
{
	tray_icon->about_callback = cb;
	tray_icon->about_callback_data = data;
}

void
gir_tray_icon_set_prefs_callback (GirTrayIcon *tray_icon,
				  void (*cb)(GirTrayIcon *tray,
					     gpointer     data),
				  gpointer data)
{
	tray_icon->prefs_callback = cb;
	tray_icon->prefs_callback_data = data;
}

void
gir_tray_icon_set_quit_callback (GirTrayIcon *tray_icon,
				 void (*cb)(GirTrayIcon *tray,
					    gpointer     data),
				 gpointer data)
{
	tray_icon->quit_callback = cb;
	tray_icon->quit_callback_data = data;
}



void
gir_tray_icon_set_active (GirTrayIcon *tray_icon)
{
	int state = tray_icon->state;

	if (state == STATE_DISCONNECTED)
		return;
	tray_icon->state = state = STATE_ACTIVE;

	gtk_image_set_from_pixbuf (tray_icon->icon_image,
				   tray_icon->pixbufs[state].pixbuf_scaled);
	if (tray_icon->active_timeout)
		g_source_remove (tray_icon->active_timeout);
	tray_icon->active_timeout =
		g_timeout_add (250, icon_active_timeout, tray_icon);
}

void
gir_tray_icon_set_connected (GirTrayIcon *tray_icon, gboolean connected)
{
	int state;

	state = connected ? STATE_INACTIVE : STATE_DISCONNECTED;
	tray_icon->state = state;
	gtk_check_menu_item_set_active (tray_icon->menu_item_connect,
					connected);
	gtk_image_set_from_pixbuf (tray_icon->icon_image,
				   tray_icon->pixbufs[state].pixbuf_scaled);
}
