/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
#include <glade/glade.h>
#include <gtk/gtk.h>
#include <string.h>
#include <stdio.h>

#include "src/gir-lirc.h"

static GladeXML *xml;
static GtkLabel *label;
static GtkWidget *version_button, *buttons_button, *controls_button;
static GtkWidget *buttons_entry;

static void
connect_check_button_toggled (GtkToggleButton *button, gpointer data)
{
	GirLirc *lirc = GIR_LIRC (data);
	gboolean active = gtk_toggle_button_get_active (button);
	
	if (active) {
		GError *error = NULL;

		if (! gir_lirc_connect (lirc, &error)) {
			/* FIXME: display the error */
			g_error_free (error);
			gtk_toggle_button_set_active (button, FALSE);
			active = FALSE;
		}
	} else
		gir_lirc_disconnect (lirc);

	gtk_widget_set_sensitive (version_button, active);
	gtk_widget_set_sensitive (controls_button, active);
	gtk_widget_set_sensitive (buttons_button, active);
	gtk_widget_set_sensitive (buttons_entry, active);
}

static void
version_button_clicked (GtkButton *button, gpointer data)
{
	GirLirc *lirc = GIR_LIRC (data);
	char *version;
	char buffer[128];

	version = gir_lirc_get_lirc_version (lirc);

	sprintf (buffer, "LIRC Version <b>%s</b>", version);
	gtk_label_set_markup (label, buffer);
	
	g_free (version);
}

static void
buttons_button_clicked (GtkButton *button, gpointer data)
{
	GirLirc *lirc = (GirLirc *)data;
	GList *list, *l;
	GString *string;
	const char *entry_text;

	entry_text = gtk_entry_get_text (GTK_ENTRY (buttons_entry));
	if (strlen (entry_text) == 0) {
		gtk_label_set_markup
			(label, "Specify the control to get button list from");
		return;
	}
		
	list = gir_lirc_get_button_list (lirc, entry_text);
	if (list) {
		string = g_string_new ("Buttons: ");

		for (l = list; l; l = l->next) {
			g_string_append_printf (string, "<b>%s</b> ",
						(char*) l->data);
			g_free (l->data);
		}
		g_list_free (list);
		gtk_label_set_markup (label, string->str);
		g_string_free (string, TRUE);
	}
}

static void
controls_button_clicked (GtkButton *button, gpointer data)
{
	GirLirc *lirc = (GirLirc *)data;
	GList *list, *l;
	GString *string;

	list = gir_lirc_get_control_list (lirc);
	if (list) {
		string = g_string_new ("Controls: ");

		gtk_entry_set_text (GTK_ENTRY (buttons_entry), list->data);

		for (l = list; l; l = l->next) {
			g_string_append_printf (string, "<b>%s</b> ",
						(char*) l->data);
			g_free (l->data);
		}
		g_list_free (list);
		gtk_label_set_markup (label, string->str);
		g_string_free (string, TRUE);
	}
}

static void
lirc_recv_ir (GirLirc *lirc,
	      int repeat_count, const char *button, const char *control,
	      gpointer data)
{
	char buffer[128];

	sprintf (buffer, "Signal <b>%s</b> <b>%s</b> <b>%d</b>",
		 control, button, repeat_count);
	gtk_label_set_markup (label, buffer);
}

#define SIGNAL(signal_name) { #signal_name, G_CALLBACK (signal_name) }
static struct {
	char *name;
	GCallback func;
} glade_signals[] = {
	SIGNAL(connect_check_button_toggled),
	SIGNAL(version_button_clicked),
	SIGNAL(controls_button_clicked),
	SIGNAL(buttons_button_clicked)
};

int
main (int argc, char *argv[])
{
	int i;
	GirLirc *lirc;
	
	gtk_init (&argc, &argv);

	lirc = gir_lirc_new ();
	xml = glade_xml_new ("test-lirc.glade", NULL, NULL);

	for (i = 0; i < G_N_ELEMENTS (glade_signals); i++) {
		glade_xml_signal_connect_data (xml, 
					       glade_signals[i].name,
					       glade_signals[i].func,
					       lirc);
	}

	g_signal_connect (G_OBJECT (lirc),
			  "recv_ir", G_CALLBACK (lirc_recv_ir), xml);

	label = GTK_LABEL (glade_xml_get_widget (xml, "label"));

	version_button = glade_xml_get_widget (xml, "version_button");
	controls_button = glade_xml_get_widget (xml, "controls_button");
	buttons_button = glade_xml_get_widget (xml, "buttons_button");
	buttons_entry = glade_xml_get_widget (xml, "buttons_entry");

	gtk_main ();

	return 0;
}
